using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Waboga.UI;

namespace Waboga
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class GameMain : Microsoft.Xna.Framework.Game
    {
        private AudioEngine audio;
        private GraphicsDeviceManager graphics;
        private ContentManager content;

        private Resources m_resources;
        private GameResources m_game_resources;

        private MainUI m_ui;

        public GameMain()
        {
#if XBOX
            audio = new AudioEngine("Content/Wagoba360.xgs");
#else
            audio = new AudioEngine("Content/Wagoba.xgs");
#endif
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);

            m_resources = new Resources(this, audio, graphics, content);
            m_game_resources = new GameResources(m_resources);

            graphics.PreferredBackBufferWidth = 1280;
            graphics.PreferredBackBufferHeight = 720;
            graphics.PreferMultiSampling = true;

            Activated += new EventHandler(GameMain_Activated);
            Deactivated += new EventHandler(GameMain_Deactivated);
            Exiting += new EventHandler(GameMain_Exiting);
        }

        private void GameMain_Exiting(object sender, EventArgs e)
        {
            if (m_ui != null)
                m_ui.Deinit();
        }

        private void GameMain_Activated(object sender, EventArgs e)
        {
            m_resources.controller.WindowActive = true;
        }

        private void GameMain_Deactivated(object sender, EventArgs e)
        {
            m_resources.controller.WindowActive = false;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            base.Initialize();
        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                m_resources.Load();
                m_ui = new MainUI(m_resources, m_game_resources);

                // Start the main music
                m_resources.sound_bank.PlayCue("Music");
            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent)
            {
                // TODO: Unload any ResourceManagementMode.Automatic content
                content.Unload();
            }

            // TODO: Unload any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            float delta_t = (float)gameTime.ElapsedGameTime.TotalSeconds;

            // Update the audio
            audio.Update();

            // Update the controller
            m_resources.controller.Update();

            // Update the board visualization
            m_game_resources.board_vis.Update(delta_t);

            // Update the UI
            m_ui.Update(delta_t);

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            m_game_resources.board_vis.Render(graphics);

            m_ui.Render();

            base.Draw(gameTime);
        }
    }
}
