using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Storage;

namespace Waboga
{
    public class Resources
    {
        public Game game;
        public AudioEngine audio;
        public GraphicsDeviceManager graphics;
        public ContentManager content_mgr;
        public StorageDevice storage_device;
        public Controller controller;

        public WaveBank wave_bank;
        public SoundBank sound_bank;

        public SpriteBatch ui_batch;

        public Model model_env_cube;
        public Model model_tile;
        public Model model_tile_goal_overlay;
        public Model model_wall;
        public Model model_wall_corner;
        public Model model_piece;
        public Model model_goal_location;

        public SpriteFont font_main;

        public Texture2D tex_white;
        public Texture2D tex_logo;
        public Texture2D tex_cursor;
        public Texture2D[] tex_wall_icons;
        public Texture2D[] tex_trophies;

        public Resources(Game _game, AudioEngine _audio, GraphicsDeviceManager _graphics, ContentManager _cmgr)
        {
            game = _game;
            audio = _audio;
            graphics = _graphics;
            content_mgr = _cmgr;
            controller = new Controller();
        }

        public void Load()
        {
            StorageDevice.BeginShowStorageDeviceGuide(PlayerIndex.One, new AsyncCallback(GetDevice), "GetDevice for Player One");
            
            // Load the assets
#if XBOX
            wave_bank = new WaveBank(audio, "Content/Wagoba360.xwb");
            sound_bank = new SoundBank(audio, "Content/Wagoba360.xsb");
#else
            wave_bank = new WaveBank(audio, "Content/Wagoba.xwb");
            sound_bank = new SoundBank(audio, "Content/Wagoba.xsb");
#endif

            ui_batch = new SpriteBatch(graphics.GraphicsDevice);

            model_env_cube = content_mgr.Load<Model>("Content/env_cube");
            model_tile = content_mgr.Load<Model>("Content/pohja");
            model_tile_goal_overlay = content_mgr.Load<Model>("Content/latka");
            model_wall = content_mgr.Load<Model>("Content/seina");
            model_wall_corner = content_mgr.Load<Model>("Content/kulma");
            model_piece = content_mgr.Load<Model>("Content/nappula");
            model_goal_location = content_mgr.Load<Model>("Content/lapyska");

            font_main = content_mgr.Load<SpriteFont>("Content/mainfont");

            tex_white = content_mgr.Load<Texture2D>("Content/white");
            tex_logo = content_mgr.Load<Texture2D>("Content/logo");
            tex_cursor = content_mgr.Load<Texture2D>("Content/cursor");
            tex_wall_icons = new Texture2D[]
            {
                MakeWallIcon(WallPiece.PieceStraight1),
                MakeWallIcon(WallPiece.PieceStraight2),
                MakeWallIcon(WallPiece.PieceStraight3),
                MakeWallIcon(WallPiece.PieceCorner),
                MakeWallIcon(WallPiece.PieceCross),
                MakeWallIcon(WallPiece.PieceS1),
                MakeWallIcon(WallPiece.PieceS2),
                MakeWallIcon(WallPiece.PieceL1),
                MakeWallIcon(WallPiece.PieceL2),
            };
            tex_trophies = new Texture2D[]
            {
                content_mgr.Load<Texture2D>("Content/cup_1"),
                content_mgr.Load<Texture2D>("Content/cup_2"),
                content_mgr.Load<Texture2D>("Content/cup_3"),
                content_mgr.Load<Texture2D>("Content/cup_4"),
            };
        }

        private void GetDevice(IAsyncResult result)
        {
            storage_device = StorageDevice.EndShowStorageDeviceGuide(result);
        }


        private Texture2D MakeWallIcon(WallPiece wall)
        {
            int bar_long = 9;
            int bar_short = 3;
            int bar_short2 = bar_short / 2;

            TexturePainter tex = new TexturePainter(graphics.GraphicsDevice, 32, 32, false);

            Color wall_col = new Color(240, 250, 130);

            // Draw the walls
            int min_x = 9999;
            int max_x = -9999;
            int min_y = 9999;
            int max_y = -9999;

            for (int i = 0; i < wall.HorizWallCount; ++i)
            {
                Point pos = wall.GetOrigHorizWallPos(i);
                PointUtil.Add(ref pos, new Point(1, 1));
                PointUtil.Scale(ref pos, bar_long);
                pos.X += bar_short2;
                pos.Y -= bar_short2;

                tex.FillRect(wall_col, pos.X, pos.Y, bar_long, bar_short);

                // Trace min/max bounds
                min_x = Math.Min(min_x, pos.X);
                max_x = Math.Max(max_x, pos.X + bar_long);
                min_y = Math.Min(min_y, pos.Y);
                max_y = Math.Max(max_y, pos.Y + bar_short);
            }
            for (int i = 0; i < wall.VertWallCount; ++i)
            {
                Point pos = wall.GetOrigVertWallPos(i);
                PointUtil.Add(ref pos, new Point(1, 1));
                PointUtil.Scale(ref pos, bar_long);
                pos.X -= bar_short2;
                pos.Y += bar_short2;

                tex.FillRect(wall_col, pos.X, pos.Y, bar_short, bar_long);

                // Trace min/max bounds
                min_x = Math.Min(min_x, pos.X);
                max_x = Math.Max(max_x, pos.X + bar_short);
                min_y = Math.Min(min_y, pos.Y);
                max_y = Math.Max(max_y, pos.Y + bar_long);
            }
            for (int i = 0; i < wall.IntersectionCount; ++i)
            {
                Point pos = wall.GetOrigIntersectPos(i);
                PointUtil.Add(ref pos, new Point(1, 1));
                PointUtil.Scale(ref pos, bar_long);
                pos.X -= bar_short2;
                pos.Y -= bar_short2;

                tex.FillRect(Color.Black, pos.X, pos.Y, bar_short, bar_short);

                // Trace min/max bounds
                min_x = Math.Min(min_x, pos.X);
                max_x = Math.Max(max_x, pos.X + bar_short);
                min_y = Math.Min(min_y, pos.Y);
                max_y = Math.Max(max_y, pos.Y + bar_short);
            }

            // Center the texture
            if (min_x < 0) min_x = 0;
            if (min_y < 0) min_y = 0;
            if (max_x > tex.Width) max_x = tex.Width;
            if (max_y > tex.Height) max_y = tex.Height;

            int move_x = (tex.Width - (max_x - min_x)) / 2 - min_x;
            int move_y = (tex.Height - (max_y - min_y)) / 2 - min_y;

            tex.Scroll(move_x, move_y);

            // Make the texture transparent
            tex.MakeColorTransparent(Color.Black);
            tex.SurroundAlphaByColor(Color.Black);
            tex.SurroundAlphaByColor(Color.Black);
            tex.SurroundAlphaByColor(Color.Black);

            return tex.Texture;
        }
    }
}
