using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;

namespace Waboga.UI
{
    public class GameAIState
    {
        public PlayerIndex plr;

        /// <summary>[player][piece] -inventory.</summary>
        public int[][] plr_inv;

        public BoardState board_state;
        public List<GameAIMove> successors = new List<GameAIMove>();

        public PlayerIndex NextPlayer
        {
            get { return (PlayerIndex)(((int)plr + 1) % plr_inv.Length); }
        }

        public GameAIState(int plrs)
        {
            plr_inv = new int[plrs][];
            for (int i = 0; i < plrs; ++i)
                plr_inv[i] = new int[(int)GamePieces.Button1];

            board_state = new BoardState();
        }

        public void CopyInventory(GameAIState from)
        {
            for (int i = 0; i < plr_inv.Length; ++i)
                Array.Copy(from.plr_inv[i], plr_inv[i], plr_inv[i].Length);
        }

        public void CopyInventory(PlayerInventory[] from)
        {
            for (int i = 0; i < plr_inv.Length; ++i)
                Array.Copy(from[i].PieceCount, plr_inv[i], plr_inv[i].Length);
        }
    }

    public class GameAIStatePool
    {
        private int m_plr_count;
        private GameAIState[] m_states;
        private int m_top;

        public GameAIStatePool(int plr_count)
        {
            m_plr_count = plr_count;
            m_states = new GameAIState[5];
            for (int i = 0; i < m_states.Length; ++i)
                m_states[i] = new GameAIState(plr_count);
        }

        public GameAIState Alloc()
        {
            if (m_top >= m_states.Length)
            {
                GameAIState[] new_states = new GameAIState[m_states.Length * 2];

                Array.Copy(m_states, new_states, m_top);
                for (int i = m_top; i < new_states.Length; ++i)
                    new_states[i] = new GameAIState(m_plr_count);

                m_states = new_states;
            }
            return m_states[m_top++];
        }

        public void Free(GameAIState state)
        {
            if (m_states[m_top - 1] != state)
                throw new Exception("Didn't free the correct state (stack rule).");
            --m_top;
        }

        public void Clear()
        {
            m_top = 0;
        }
    }
}
