using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Storage;

namespace Waboga.UI
{
    public class GameStartParams
    {
        public bool has_computer_ai = false;
        public int players = 2;
        public int pieces_per_player = 1;

        public int[] piece_counts = new int[(int)GamePieces.Button1];

        public GameStartParams()
        {
            piece_counts[(int)GamePieces.PieceStraight2] = 10;
            piece_counts[(int)GamePieces.PieceCorner] = 5;
        }

        public void TryLoad(StorageDevice dev)
        {
            if (dev == null || !dev.IsConnected)
                return;

            try
            {
                using (StorageContainer cont = dev.OpenContainer("Waboga"))
                {
                    string fpath = Path.Combine(cont.Path, "save.dat");
                    using (BinaryReader fin = new BinaryReader(File.Open(fpath, FileMode.Open)))
                    {
                        has_computer_ai = fin.ReadBoolean();
                        players = fin.ReadInt32();
                        pieces_per_player = fin.ReadInt32();

                        for (int i = 0; i < piece_counts.Length; ++i)
                            piece_counts[i] = fin.ReadInt32();
                    }
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("Error loading:\n" + e);
            }
        }

        public void Save(StorageDevice dev)
        {
            if (dev == null || !dev.IsConnected)
                return;

            try
            {
                using (StorageContainer cont = dev.OpenContainer("Waboga"))
                {
                    string fpath = Path.Combine(cont.Path, "save.dat");
                    using (BinaryWriter fout = new BinaryWriter(File.Open(fpath, FileMode.Create)))
                    {
                        fout.Write(has_computer_ai);
                        fout.Write((int)players);
                        fout.Write((int)pieces_per_player);

                        for (int i = 0; i < piece_counts.Length; ++i)
                            fout.Write((int)piece_counts[i]);
                    }
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("Error saving:\n" + e);
            }
        }
    }
}
