using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Waboga;

namespace Waboga.UI
{
    public class MainMenuPage : UIPage
    {
        private const float LogoYHidden = -500.0f;
        private const float LogoYShown = -20.0f;

        private string m_version = null;
        private string Version
        {
            get
            {
                if (m_version == null)
                    m_version = Assembly.GetExecutingAssembly().GetName().Version.ToString();
                return m_version;
            }
        }

        private float m_logo_y = LogoYHidden;

        public override void Update(MainUI ui, float delta_t)
        {
            UIRenderer renderer = UIBegin();

            m_logo_y = MathHelper.Lerp(m_logo_y, IsTopmost ? LogoYShown : LogoYHidden, 0.09f);
            renderer.DrawIconAtCentered(m_res.tex_logo, m_logo_y, 0.8f);

            if (IsTopmost)
            {
                renderer.WindowStartCentered(360, 230, 210);

                renderer.WriteLineCentered("Main menu", Color.White, 2.0f);

                if (UIButton("btn_start_game", "Start Game"))
                {
                    ui.PushPage(new GameSetupPage());
                }

                if (UIButton("btn_help", "Instructions"))
                {
                    ui.PushPage(new HelpPage());
                }

                if (UIButton("btn_credits", "Credits"))
                {
                    ui.PushPage(new CreditsPage());
                }

                if (UIButton("btn_quit_game", "Quit game") ||
                    Controller.WasPressed(ControllerButton.KeyEsc))
                {
                    m_res.game.Exit();
                }

                renderer.WindowEnd();

                renderer.WriteAt(new Vector2(30, 550), Version, Color.White, 0.25f);
            }

            UIEnd();
        }
    }
}
